local AddonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

a.EquipmentSets = {}

------------------------------------------------------------------------ Common
a.spells["Commanding Shout"] = {
    ID = 469,
    Buff = c.STAMINA_BUFFS,
    BuffUnit = "player",
    EarlyRefresh = 5,
    FlashColor = "yellow",
    CheckFirst = function()
        return not s.MyBuff(c.GetID("Battle Shout"), "player")
    end
}

a.spells["Commanding Shout for Rage"] = {
    ID = 469,
    CheckFirst = function()
        return s.PowerMissing("player") >= 20 + 5 * s.TalentRank(12835) -- Booming Voice
            and not s.MyBuff(c.GetID("Battle Shout"), "player")
    end
}

a.spells["Battle Shout"] = {
    ID = 6673,
    Buff = c.STR_AGI_BUFFS,
    BuffUnit = "player",
    EarlyRefresh = 5,
    FlashColor = "yellow",
    CheckFirst = function()
        return not s.MyBuff(c.GetID("Commanding Shout"), "player")
    end
}

a.spells["Battle Shout for Rage"] = {
    ID = 6673,
    CheckFirst = function()
        return s.PowerMissing("player") >= 20 + 5 * s.TalentRank(12835) -- Booming Voice
            and not s.MyBuff(c.GetID("Commanding Shout"), "player")
    end
}

a.spells["Heroic Throw"] = {
    ID = 57755,
}

a.spells["Rend"] = {
    ID = 772,
    MyDebuff = 772,
}

a.spells["Heroic Strike over 65"] = {
    ID = 78,
    FlashID = { 78, 845 }, -- Cleave
    NoGCD = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return s.Power("player") >= 65
    end
}

a.spells["Recklessness"] = {
    ID = 1719,
    NoGCD = 1,
    FlashColor = "yellow",
}

a.spells["Berserking"] = {
    ID = 26297,
    NoGCD = 1,
    FlashColor = "yellow",
}

a.spells["Berserker Rage"] = {
    ID = 18499,
    NoGCD = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return s.Power("player") < 75
    end
}

a.spells["Pummel"] = {
    ID = 6552,
    FlashColor = "aqua",
    Interrupt = 1,
}

--------------------------------------------------------------------------- Dps
a.spells["Execute"] = {
    ID = 5308,
}

a.spells["Colossus Smash"] = {
    ID = 86346,
    MyDebuff = 86346,
}

a.spells["Enraged Regeneration below 48"] = {
    ID = 55694,
    NoGCD = 1,
    FlashColor = "red",
    CheckFirst = function()
        return s.HealthPercent("player") < 48
    end
}

a.spells["Heroic Strike on Battle Trance"] = {
    ID = 78,
    FlashID = { 78, 845 }, -- Cleave
    NoGCD = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return c.HasBuff("Battle Trance")
    end
}

-------------------------------------------------------------------------- Arms
a.spells["Overpower"] = { ID = 7384 }
a.spells["Taste for Blood"] = { ID = 60503 }
a.spells["Slaughter"] = { ID = 84586 }
a.spells["Battle Trance"] = { ID = 12964 }

a.spells["Slam"] = {
    ID = 1464,
    CheckFirst = function()
        local rage = s.Power("player")
        if c.IsCasting("Slam") then
            rage = rage - s.SpellCost(1464)
        end
        return rage >= 35 or s.Buff("Deadly Calm", "player")
    end
}

a.spells["Mortal Strike"] = {
    ID = 12294,
}

a.spells["Overpower"] = {
    ID = 7384,
}

a.spells["Deadly Calm"] = {
    ID = 85730,
    NoGCD = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return s.Power("player") < 40 and c.GetBuffStack("Slaughter") == 3
    end
}

a.spells["Heroic Strike on Deadly Calm"] = {
    ID = 78,
    FlashID = { 78, 845 }, -- Cleave
    NoGCD = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return c.HasBuff("Deadly Calm")
    end
}

a.spells["Inner Rage"] = {
    ID = 1134,
    NoGCD = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return c.GetCooldown("Deadly Calm", true) > 15
    end
}

--a.spells["Recklessness for Arms"] = {
--    ID = 1719,
--    NoGCD = 1,
--    FlashColor = "yellow",
--    CheckFirst = function()
--        return s.Power("player") > 60
--            and not s.Buff("Enrage", "player")
--            and s.SpellCooldown("Mortal Strike") < 1.5
--    end
--}

a.spells["Berserker Rage without Deadly Calm"] = {
    ID = 18499,
    NoGCD = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return s.Power("player") < 75 and not c.HasBuff("Deadly Calm")
    end
}

a.spells["Battle Stance"] = {
    Type = "form",
    ID = 2457,
    FlashColor = "yellow",
}

a.spells["Berserking without Deadly Calm"] = {
    ID = 26297,
    NoGCD = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return not c.HasBuff("Deadly Calm")
    end
}

local function dancable()
    return s.Power("player") <= 25 + s.TalentRank(12676) * 25
        or c.GetBuffDuration("Deadly Calm") > 3
end

a.spells["Dance to Battle Stance"] = {
    Type = "form",
    ID = 2457,
    NoGCD = 1,
    CheckFirst = function()
        return a.GetConfig("dance")
            and (s.Power("player") <= 75
                or c.GetBuffDuration("Deadly Calm") > 3)
            and (c.HasBuff("Overpower")
                or c.HasBuff("Taste for Blood")
                or not c.HasMyDebuff("Rend"))
    end
}

a.spells["Dance to Berserker Stance"] = {
    Type = "form",
    ID = 2458,
    NoGCD = 1,
    CheckFirst = function()
        return a.GetConfig("dance")
            and (s.Power("player") <= 75
                or c.GetBuffDuration("Deadly Calm") > 3)
            and not c.HasBuff("Overpower")
            and not c.HasBuff("Taste for Blood")
            and c.HasMyDebuff("Rend")
    end
}

-------------------------------------------------------------------------- Fury
a.spells["Bloodsurge"] = { ID = 46916 }

a.spells["Berserker Rage for Rage Effect"] = {
    ID = 18499,
    NoGCD = 1,
    FlashColor = "yellow",
    CheckFirst = function()
        return c.GetCooldown("Raging Blow") == 0
            and not s.UsableSpell(c.GetID("Raging Blow"))
    end
}

a.spells["Death Wish"] = {
    ID = 12292,
    NoGCD = 1,
    FlashColor = "yellow",
}

a.spells["Raging Blow"] = {
    ID = 85288,
}

a.spells["Instant Slam"] = {
    ID = 1464,
    CheckFirst = function()
        return c.HasBuff("Bloodsurge")
    end
}

a.spells["Bloodthirst"] = {
    ID = 23881,
}

a.spells["Victory Rush"] = {
    ID = 34428,
}

a.spells["Execute for Buff"] = {
    ID = 5308,
    Buff = 90806, -- Executioner
    BuffUnit = "player",
    Stack = 5,
    EarlyRefresh = 2,
}

a.spells["Whirlwind"] = {
    ID = 1680,
}

a.spells["Berserker Stance"] = {
    Type = "form",
    ID = 2458,
    FlashColor = "yellow",
}

-------------------------------------------------------------------- Protection
a.spells["Thunderstruck"] = { ID = 87096 }

a.EquipmentSets.ProtT13 = {
    HeadSlot = { 78784, 76990, 78689 },
    ShoulderSlot = { 78829, 76992, 78734 },
    ChestSlot = { 78753, 76988, 78658 },
    HandsSlot = { 78764, 76989, 78669 },
    LegsSlot = { 78800, 76991, 78705 },
}

a.spells["Shield Slam"] = {
    ID = 23922,
}

a.spells["Revenge"] = {
    ID = 6572,
}

a.spells["2pT13 Revenge"] = {
    ID = 6572,
    CheckFirst = function()
        return c.WearingSet(2, "ProtT13")
    end
}

a.spells["Devastate"] = {
    ID = 20243
}

a.spells["Devastate to 3"] = {
    ID = 20243,
    Debuff = c.ARMOR_DEBUFFS,
    Stack = 3
}

a.spells["Thunder Clap"] = {
    ID = 6343
}

a.spells["Thunder Clap for Debuff"] = {
    ID = 6343,
    Debuff = c.SLOW_MELEE_DEBUFFS,
    EarlyRefresh = 2,
}

a.spells["Thunder Clap for Rend"] = {
    ID = 6343,
    CheckFirst = function()
        local t = c.GetMyDebuffDuration("Rend")
        return t > .2 and t < 3
    end
}

a.spells["Shockwave"] = {
    ID = 46968,
}

a.spells["Shockwave at 3 Stacks"] = {
    ID = 46968,
    CheckFirst = function()
        return c.GetBuffStack("Thunderstruck") == 3
    end
}

a.spells["Demoralizing Shout"] = {
    ID = 1160,
    Debuff = c.PHYSICAL_DAMAGE_DEBUFFS,
    EarlyRefresh = 2,
}

a.spells["Shield Block"] = {
    ID = 2565,
    NoGCD = 1,
    FlashColor = "yellow",
}

a.spells["Last Stand"] = {
    ID = 12975,
    NoGCD = 1,
    FlashColor = "red",
    CheckFirst = function()
        return s.HealthPercent("player") < 25
    end
}

a.spells["Enraged Regeneration"] = {
    ID = 55694,
    FlashColor = "yellow",
}

a.spells["Shield Wall"] = {
    ID = 871,
    NoGCD = 1,
    FlashColor = "yellow",
}

a.spells["Defensive Stance"] = {
    Type = "form",
    ID = 71,
    FlashColor = "yelloW",
}

a.spells["Rend unless Refresh Pending"] = {
    ID = 772,
    MyDebuff = 772,
    CheckFirst = function()
        return not c.IsAuraPendingFor("Thunder Clap")
    end
}

a.spells["Elusive"] = { ID = 107951 }
a.spells["Fire of the Deep"] = {
    Type = "item",
    ID = 77117,
    Buff = "Elusive",
    FlashColor = "yellow",
}

a.spells["Vigilance"] = {
    ID = 50720,
    FlashColor = "yellow",
    Override = function()
        if not s.InRaidOrParty() then
            return false
        end
        
        if not a.VigilanceTarget then
            return true
        end
        
        if s.InCombat() then
            return not s.MyBuff(50720, a.VigilanceTarget)
        else
            return s.MyBuffDuration(50720, a.VigilanceTarget) < 5 * 60
        end
    end
}

a.spells["Taunt"] = {
    ID = 355,
    NoGCD = true,
    CheckFirst = c.CheckFirstForTaunts,
}

a.spells["Challenging Shout"] = {
    ID = 1161,
    CheckFirst = c.CheckFirstForTaunts,
}
